package Data::Squirrel;

use strict;
use vars qw($VERSION @ISA $VERBOSE);

$VERSION = '0.01';

use Apache::Session;
use Carp;
@ISA=qw(Apache::Session);

sub DEBUG {0}
$VERBOSE=0;

###########################################################
sub TIEHASH 
{
    my($package, $args) = @_;
    $args ||= {};

    # Make sure that the arguments to tie make sense
    
    if(ref $args ne "HASH") {
        die "Additional arguments should be in the form of a hash reference";
    }

    #Set-up the data structure and make it an object
    #of our class
    my $self = {
        args         => $args,
        lock         => 0,
        status       => 0,
        lock_manager => undef,  # These two are object refs ...
        object_store => undef,
        generate     => undef,  # but these three are subroutine refs
        serialize    => undef,
        unserialize  => undef,
    };
    
    bless $self, $package;
    $self->populate();
    return $self;
}

###########################################################
sub FETCH 
{
    my($self, $key) = @_;
    local $self->{data}={};
    local $self->{serialized};
    $self->{data}{_session_id}=$key;        
    return unless $self->{object_store}->exists($self);

    $self->acquire_read_lock;
    eval {$self->{object_store}->materialize($self)};
    DEBUG and $@ and warn $@;
    $self->release_read_lock;
    $self->{unserialize}->($self);
    return $self->{data}{DATA};
}

###########################################################
sub STORE 
{
    my($self, $key, $data) = @_;
    local $self->{data}={DATA=>$data};
    local $self->{serialized};
    $self->{data}{_session_id}=$key;                    
    $self->{data}{TIME}=time;

    $self->{serialize}->($self);

    $self->acquire_write_lock;
    eval {
        if($self->{object_store}->exists($self)) {
            $self->{object_store}->update($self);
        } else {
            $self->{object_store}->insert($self);
        }
    };
    $self->release_write_lock;
    die $@ if $@;

    return $self->{data}{DATA};
}

###########################################################
sub DELETE 
{
    my($self, $key)=@_;
    local $self->{data}={};
    $self->{data}{_session_id}=$key;
    return unless $self->{object_store}->exists($self);

    local $self->{serialized};
    $self->FETCH($key);     # get object into {data}{DATA}

    $self->acquire_write_lock;
    eval { 
        $self->{object_store}->remove($self);
    };
    $self->release_write_lock;
    die $@ if $@;

    return $self->{data}{DATA};

}

###########################################################
sub EXISTS 
{
    my($self, $key)=@_;
    local $self->{data}={};
    DEBUG and warn "Looking for $key\n";
    $self->{data}{_session_id}=$key;        
    $self->acquire_read_lock;
    my $ret=$self->{object_store}->exists($self);
    $self->release_read_lock;
    DEBUG and warn $ret ? "Found\n" : "Not found\n";
    return $ret;
}

###########################################################
sub FINDFIRST
{
    my($self)=@_;
    croak ref($self), "FINDFIRST not implemented";
}

###########################################################
sub FINDNEXT
{
    my($self)=@_;
    croak ref($self), "::FINDNEXT not implemented";
}

###########################################################
sub DESTROY 
{
    my($self) = @_;
    
    $self->release_all_locks;
}

sub expire 
{
    my($self, $expire)=@_;

    unless(defined $expire) {
        warn "Must specify expiry time";
        return;
    }


    #####################################################
    # clean up the object store
    my $ids=$self->list_ids();
    my $now = time;
    DEBUG and warn "It is now $now\n";
    foreach my $id (@$ids) { 

        local $self->{data};
        $self->{data}{_session_id}=$id;

        DEBUG and warn "Checking $id\n";

        $self->acquire_read_lock;
        eval {
            local $self->{serialized};                  # get storage date
            $self->{object_store}->materialize($self);  
            $self->{unserialize}->($self);
        };
        if($@) {
            warn $@;
            $self->release_read_lock;
            next;
        }
        next unless $self->{data} and defined $self->{data}{TIME};                            
        DEBUG and warn "Created at $self->{data}{TIME}\n";
        next unless ($now - $self->{data}{TIME}) > $expire;
        DEBUG and warn "Too old ($expire)\n";

        $self->acquire_write_lock;                      # delete if too old
        eval { 
            $VERBOSE and warn "Expiring $id";
            $self->{object_store}->remove($self);
        };
        warn $@ if $@;
        $self->release_write_lock;
        $self->release_read_lock;
    }

    #####################################################
    # clean up the lock manager
    if($self->{lock_manager}->can('clean')) {
        my $directory = $self->{args}->{LockDirectory} || $Apache::Session::Lock::File::Directory;
        die "The directory for object storage must be passed as an argument or set in httpd.conf" 
                unless defined $directory;
        $self->{lock_manager}->clean($directory, $expire);
    } 
    elsif($self->{lock_manager}->can('expire')) {
        $self->{lock_manager}->expire($expire);
    }

}



1;
__END__
# Below is the stub of documentation for your module. You better edit it!

=head1 NAME

Data::Squirrel - Store arbitrary data anywhere Apache::Session can

=head1 SYNOPSIS

  use Data::Squirrel;
  tie %squirrel, "Data::Squirrel::MySQL", {DataSource=>"dbi:mysql:test", 
          UserName=>"test", Password=>"test", 
            LockDataSource=>"dbi:mysql:test", 
            LockUserName=>"test", LockPassword=>"test", };

  $squirrel{$id}=$somedata;



=head1 DESCRIPTION

There is no documentation right now.  Look at the source code.  Look at
test.pl.  Good luck.


=head1 AUTHOR

Philip "Leolo" Gwyn, fil@pied.nu

=head1 SEE ALSO

Apache::Session, perl(1).

=cut
