#############################################################################
#
# Data::Squirrel::Store::GDBM_File
# Implements session object storage via Perl's GDBM_File module
# Copyright(c) 2000 Jeffrey William Baker (jwbaker@acm.org)
# Distribute under the Artistic License
#
############################################################################

package Data::Squirrel::Store::GDBM_File;

use strict;
use vars qw($VERSION);
use GDBM_File;

sub DEBUG {0}

$VERSION = '1.00';

sub new {
    my $class = shift;
    
    return bless {dbm => {}}, $class;
}

sub tie_me
{
    my ($self, $file)=@_;

    if (!tied %{$self->{dbm}}) {
        my $rv = tie %{$self->{dbm}}, 'GDBM_File', 
                            $file, &GDBM_WRCREAT, 0640;

        if (!$rv) {
            die "Could not open $file: $!";
        }
    }
}

sub untie_me
{
    my ($self)=@_;

    return unless tied %{$self->{dbm}};
    untie %{$self->{dbm}};
}

sub insert {
    my $self    = shift;
    my $session = shift;

    $self->tie_me($session->{args}{FileName});
    if (exists $self->{dbm}->{$session->{data}->{_session_id}}) {
        die "Object already exists in the data store";
    }
    
    $self->{dbm}->{$session->{data}->{_session_id}} = $session->{serialized};
    $self->untie_me;
}

sub update {
    my $self = shift;
    my $session = shift;
    
    $self->tie_me($session->{args}{FileName});    
    $self->{dbm}->{$session->{data}->{_session_id}} = $session->{serialized};
    $self->untie_me;
}

sub materialize {
    my $self = shift;
    my $session = shift;
    
    $self->tie_me($session->{args}{FileName});    
    DEBUG and warn "Reading $session->{data}->{_session_id}";
    $session->{serialized} = $self->{dbm}->{$session->{data}->{_session_id}};
    DEBUG and warn "Read ", length($session->{serialized}), " bytes.";

    if (!defined $session->{serialized}) {
        die "Object does not exist in data store";
    }
    $self->untie_me;
}

#########################################################
sub _remove {
    my $self = shift;
    my $session = shift;
    
    $self->tie_me($session->{args}{FileName});
    delete $self->{dbm}->{$session->{data}->{_session_id}};
    $self->untie_me;
}

sub remove
{
    my($self, $squirrel) = @_;

    eval {$self->_remove($squirrel);};
    warn $@ if $@;
}

#########################################################
sub exists
{
    my($self, $squirrel) = @_;

    $self->tie_me($squirrel->{args}{FileName});
    my $ret=exists $self->{dbm}{$squirrel->{data}{_session_id}};
    $self->untie_me;
    return $ret;
}

#########################################################
sub list_ids
{
    my($self, $squirrel) = @_;
    
    $self->tie_me($squirrel->{args}->{FileName});
    my @ids=keys %{$self->{dbm}};
    $self->untie_me;
    return @ids if wantarray;
    return \@ids;
}


1;

=pod

=head1 NAME

Data::Squirrel::Store::GDBM_File - Use GDBM_File to store persistent objects

=head1 SYNOPSIS

 use Data::Squirrel::Store::GDBM_File;
 
 my $store = new Data::Squirrel::Store::GDBM_File;
 
 $store->insert($ref);
 $store->update($ref);
 $store->materialize($ref);
 $store->remove($ref);

=head1 DESCRIPTION

This module fulfills the storage interface of Data::Squirrel.  The serialized
objects are stored in a Berkeley DB file using the GDBM_File Perl module.  If
GDBM_File works on your platform, this module should also work.

=head1 OPTIONS

This module requires one argument in the usual Data::Squirrel style.  The
name of the option is FileName, and the value is the full path of the database
file to be used as the backing store.  If the database file does not exist,
it will be created.  Example:

 tie %s, 'Data::Squirrel::GDBM_File', undef,
    {FileName => '/tmp/sessions'};

=head1 AUTHOR

This module was written by Jeffrey William Baker <jwbaker@acm.org>.

=head1 SEE ALSO

L<Data::Squirrel>, L<GDBM_File>
